searchData={"items":[{"type":"behaviour","title":"AOC","doc":"Advent of Code solution module macro and helpers.\n\nThis module contains the `aoc/3` macro, which should be used to write a solution module for a\ngiven advent of code challenge. The intended use is to write your solution for day ` `, year\n` ` as follows:\n\n```\nimport AOC\n\naoc  ,   do\n  def p1(input) do\n    # Part 1 solution goes here\n  end\n\n  def p2(input) do\n    # Part 1 solution goes here\n  end\nend\n```\n\nWriting a solution module with the `aoc/3` macro enables you to use the functions defined in the\n`AOC.IEx` module to test your solutions with ease. For instance, you can use `AOC.IEx.p1e/1` to\ncall `p1` with the example input of the current day and `AOC.IEx.p1i/1` to call `p1` with the\npuzzle input of the current day. Similar functions are available for `p2`.\n\nNote that the code skeleton shown above can be generated by running `mix aoc.gen` or `mix aoc`.","ref":"AOC.html"},{"type":"macro","title":"AOC.aoc/3","doc":"Generate an advent of code solution module for a given year and day.\n\nThe generated module will be named `Y .D `. The helpers in `AOC.IEx` rely on this\nconvention to find your solution module.","ref":"AOC.html#aoc/3"},{"type":"macro","title":"Examples - AOC.aoc/3","doc":"```\nimport AOC\n\naoc 2020, 1 do\n  def some_function do\n    :foo\n  end\nend\n```\n\nis equivalent to:\n\n```\ndefmodule Y2020.D1 do\n  @behaviour AOC\n\n  def some_function do\n    :foo\n  end\nend\n```","ref":"AOC.html#aoc/3-examples"},{"type":"macro","title":"AOC.aoc_test/4","doc":"Generate an advent of code test module for a given year and day.\n\nThe generated module will be named `Y .D .AOCTest`. It will be tagged with the year,\nday and date of the puzzle, and will contain helper functions, `input_path/0`, `example_path/0`,\n`input_string/0` and `example_string/0` which can be used to access the example and puzzle\ninput, as described in `AOC.Case`.\n\nThe generated module will import the solution module of the same date (unless `import?: false`\nis provided as an option) and automatically calls `ExUnit.DocTest.doctest/1` on the solution\nmodule, unless `doctest?: false` is provided as an option. Any other options (such as `async:\ntrue`) are passed to ExUnit.","ref":"AOC.html#aoc_test/4"},{"type":"macro","title":"Examples - AOC.aoc_test/4","doc":"```\nimport AOC\n\naoc_test 2020,1, async: true do\n  test \"does my helper work?\" do\n    assert some_helper(:foo) == 42\n  end\nend\n```\n\nIs equivalent to:\n\n```\ndefmodule Y2020.D1.AOCTest do\n  use AOC.Case, year: 2020, day: 1, async: true\n\n  import Y2020.D1\n\n  test \"does my helper work?\" do\n    assert some_helper(:foo) == 42\n  end\n\n  doctest Y2020.D1\nend\n```","ref":"AOC.html#aoc_test/4-examples"},{"type":"callback","title":"AOC.p1/1","doc":"Part 1 solution.\n\nMust accept a string which represents the puzzle or example input.","ref":"AOC.html#c:p1/1"},{"type":"callback","title":"AOC.p2/1","doc":"Part 2 solution.\n\nMust accept a string which represents the puzzle or example input. This callback is marked as\noptional as day 25 does not have a second part; this also prevents warnings while working on\npart 1.","ref":"AOC.html#c:p2/1"},{"type":"module","title":"AOC.Case","doc":"`ExUnit.CaseTemplate` for writing advent of code test cases.\n\nThis template inserts module tags and helper functions into the unit test module. Concretely, it\ninserts several module tags, which can be used to selectively test certain days using `mix\ntest`. It also inserts several helpers which make it possible to access the example and puzzle\ninput within the tests.\n\nThe `use` statement expects a `year` and `day` option, which are used to generate the correct\nmodule tags and helper functions.\n\nConsider using `AOC.aoc_test/4` instead of this case template.","ref":"AOC.Case.html"},{"type":"module","title":"Module tags and filtering - AOC.Case","doc":"This case adds the following module tags:\n\n* `year`: the year of the puzzle\n* `day`: the day of the puzzle\n* `aoc`: the year and day of the puzzle, separated by a dash, e.g. `\"1991-8\"`\n\n`mix test` allows you to filter tests based on tags, therefore, you can do the following:\n\n* `mix test --only year:1991`: will run all tests associated with year 1991.\n* `mix test --only day:8`: will run all tests associated with day 8.\n\nSpecifying multiple `--only` options _combines_ the options, i.e., running\n`mix test --only day:8 --year:1991` will run all tests of year 1991 _and_ all the tests of any\nday 8. This is problematic if your solution repository spans multiple years. Therefore, we\ninclude the `aoc` tag, which uniquely identifies a day:\n\n* `mix test --only aoc:1991-8`: will run all the tests associated with day 8 of year 1991.\n\nIf your solution repository only contains modules for a single year, this behaves identical to\n`mix test --only day:8`.","ref":"AOC.Case.html#module-module-tags-and-filtering"},{"type":"module","title":"Helpers - AOC.Case","doc":"This template injects several helpers into the test module which can be used to access the\nexample and puzzle input. These helpers behave similar to those defined in `AOC.IEx`, but always\nfetch the input or example associated with the solution module. The following helpers are\ninjected:\n\n* `example_string/0`: Fetch the example input with trailing newlines removed.\n* `input_string/0`: Fetch the puzzle input, trailing newlines are removed.\n* `input_path/0`: Get the path to the puzzle input.\n* `example_path/0`: Get the path to the example input.","ref":"AOC.Case.html#module-helpers"},{"type":"module","title":"AOC.IEx","doc":"IEx helpers for advent of code.\n\nThis module contains various helpers that make it easy to call procedures in your solution\nmodules. It is intended to be used while testing solutions in iex, the elixir shell.\n\nIn order to avoid prefixing all calls with `AOC.IEx`, we recommend adding `import AOC.IEx` to\nyour [`.iex.exs` file](https://hexdocs.pm/iex/IEx.html#module-the-iex-exs-file).","ref":"AOC.IEx.html"},{"type":"module","title":"Requirements and `AOC.aoc/3` - AOC.IEx","doc":"In order to find a module for a given day and year, this module expects the module to have the\nname `Y .D `. This is always the case if the `AOC.aoc/3` macro was used to build the\nsolution module (and thus the case if the template generated by `mix aoc.gen` or `mix aoc` is\nused).\n\nFurthermore, it is expected that the solutions for part 1 and part 2 are defined in non-private\nfunctions named `p1` and `p2`. These functions must accept one argument: the puzzle input,\nrepresented as a string.","ref":"AOC.IEx.html#module-requirements-and-aoc-aoc-3"},{"type":"module","title":"Functions in this module - AOC.IEx","doc":"This module provides the `p1e/1`, `p1i/1`, `p2e/1` and `p2i/1` functions, which call your part\none or part two solution with the example (`p1e/1`, `p2e/1`) or puzzle input (`p1i/1`, `p2i/1`)\nfrom within iex.  You can also use `p1/2` and `p2/2` to call the `p1` and `p2` functions of your\nsolution module directly.\n\n`mod/1` can be used to obtain the current solution module, which is useful if you wish to test\nother functions in your solution module. Moreover, `example_path/1`, `input_path/1`,\n`example_string/1`, and `input_string/1`, can be used to experiment with the puzzle input and\nexample input retrieved by `mix aoc.get` or `mix aoc` inside iex.","ref":"AOC.IEx.html#module-functions-in-this-module"},{"type":"module","title":"Specifying the puzzle date - AOC.IEx","doc":"The functions in this module all select a puzzle (or more specifically, its solution module,\ninput or example) based on the current time. For instance, the `p1/2` function calls the `p1`\nfunction of the solution module that corresponds to the current day. The current day (and year)\nis determined by `NaiveDateTime.local_now/0`, or by `DateTime.now/2` if a time zone was set, as\ndescribed in the [README](readme.html#time-zones). If it is past midnight, or if you wish to\nsolve an older challenge, there are a few options at your disposal:\n\n- Each function in this module accepts an optional keyword list through which the year and day\n  can be specified. For instance, if you wish to run part 1 of of 8 december 1991, you could\n  write the following code: `p1( , year: 1991, day: 8)`. If you omit the day or year, the\n  current day or year is used by default. `p1( , day: 8)` would, for instance, call part 1\n  of day 8 of the current year.\n\n- The year and day can be configured through the `:advent_of_code_utils` application\n  environment. For instance, you can set the year to `1991` and the day to `8` by placing the\n  following in your `config/config.exs`:\n\n  ```elixir\n  import Config\n\n  config :advent_of_code_utils,\n    day: 8,\n    year: 1991\n  ```\n\nBoth of these options can be combined. You can, for instance, set the year in\n`config/config.exs` and select the day when calling `p1/2` or any other function in this module.\n\nTo summarise, the day or year is determined according to the following rules:\n\n1. If year or day is passed as part of the keyword list argument, it is always used.\n2. If `:year` or `:day` is present in the `:advent_of_code_utils` application environment, it is\nused.\n3. The `year` or `day` returned by `NaiveDateTime.local_now/0` or `DateTime.now/2` is used.","ref":"AOC.IEx.html#module-specifying-the-puzzle-date"},{"type":"module","title":"Automatic recompilation - AOC.IEx","doc":"It is often necessary to recompile the current `mix` project before running code. To avoid\nrepeatedly calling `IEx.Helpers.recompile/1`, the various `p*` functions in this module and\n`mod/1` will automatically recompile the current mix project (with `IEx.Helpers.recompile/1`)\nwhen `:auto_compile?` is set to `true` in the `:advent_of_code_utils` application environment:\n\n```\nimport Config\n\nconfig :advent_of_code_utils, auto_compile?: true\n```","ref":"AOC.IEx.html#module-automatic-recompilation"},{"type":"module","title":"Elapsed time - AOC.IEx","doc":"Some developers are interested in the runtime of their solutions. When the `time_calls?` options\nis set in the `:advent_of_code_utils` application environment, the runtime of a solution will be\nshown when calling `p1/2`, `p2/2`, `p1i/1`, `p1e/1`, `p2i/1` and `p2e/1`. By default, this\nfeature is disabled.\n\n```\nimport Config\n\nconfig :advent_of_code_utils, time_calls?: true\n```\n\nThis feature can also be enabled or disabled by passing `time: true` or `time: false` as an\noption to `p1/2`, `p2/2`, `p1i/1`, `p1e/1`, `p2i/1` or `p2e/1`.","ref":"AOC.IEx.html#module-elapsed-time"},{"type":"function","title":"AOC.IEx.example_path/1","doc":"Obtain the path of the example input of the current puzzle.\n\nIf not present in the options list, `day` and `year` are fetched from the application\nenvironment or based on the local time. Refer to the module documentation for additional\ninformation.","ref":"AOC.IEx.html#example_path/1"},{"type":"function","title":"AOC.IEx.example_string/1","doc":"Obtain the example input of the current puzzle as a string.\n\nTrailing newlines are stripped from the example input string.\n\nIf not present in the options list, `day` and `year` are fetched from the application\nenvironment or based on the local time. Refer to the module documentation for additional\ninformation.","ref":"AOC.IEx.html#example_string/1"},{"type":"function","title":"AOC.IEx.input_path/1","doc":"Obtain the path of the input for the current puzzle.\n\nIf not present in the options list, `day` and `year` are fetched from the application\nenvironment or based on the local time. Refer to the module documentation for additional\ninformation.","ref":"AOC.IEx.html#input_path/1"},{"type":"function","title":"AOC.IEx.input_string/1","doc":"Obtain the puzzle input of the current puzzle as a string.\n\nTrailing newlines are stripped from the puzzle input string.\n\nIf not present in the options list, `day` and `year` are fetched from the application\nenvironment or based on the local time. Refer to the module documentation for additional\ninformation.","ref":"AOC.IEx.html#input_string/1"},{"type":"function","title":"AOC.IEx.mod/1","doc":"Get the module name for the currently configured puzzle.\n\nIf not present in the options list, `day` and `year` are fetched from the application\nenvironment or based on the local time. Refer to the module documentation for additional\ninformation.\n\nThis function may cause recompilation if `auto_compile?` is enabled.","ref":"AOC.IEx.html#mod/1"},{"type":"function","title":"Examples - AOC.IEx.mod/1","doc":"iex> mod(year: 1991, day: 8)\n    Y1991.D8\n    iex> Application.put_env(:advent_of_code_utils, :year, 1991)\n    iex> Application.put_env(:advent_of_code_utils, :day, 8)\n    iex> mod()\n    Y1991.D8\n    iex> mod(day: 9)\n    Y1991.D9\n    iex> mod(year: 2000)\n    Y2000.D8\n    iex> Application.put_env(:advent_of_code_utils, :year, 2000)\n    iex> Application.put_env(:advent_of_code_utils, :day, 3)\n    iex> mod()\n    Y2000.D3","ref":"AOC.IEx.html#mod/1-examples"},{"type":"function","title":"AOC.IEx.p1/2","doc":"Call part 1 of the current puzzle with the given input.\n\nCalls, `Y .D .p1/1` with `input`.\n\nIf not present in the options list, `day` and `year` are fetched from the application\nenvironment or based on the local time. Refer to the module documentation for additional\ninformation.\n\nThe `time:` option can be used to control if the execution of `p1` will be timed. If it is not\npresent, the value of `:time_calls?` in the application environment (default: `false`) will be\nrespected.\n\nThis function may cause recompilation if `auto_compile?` is enabled.","ref":"AOC.IEx.html#p1/2"},{"type":"function","title":"AOC.IEx.p1e/1","doc":"Call part 1 of the current puzzle with its example input.\n\nUses `p1/2` and `example_string/1` to call `Y .D .p1/1` with the example input of\n`year` and `day`.\n\nIf not present in the options list, `day` and `year` are fetched from the application\nenvironment or based on the local time. Refer to the module documentation for additional\ninformation.\n\nThe `time:` option can be used to control if the execution of `p1` will be timed. If it is not\npresent, the value of `:time_calls?` in the application environment (default: `false`) will be\nrespected.\n\nThis function may cause recompilation if `auto_compile?` is enabled.","ref":"AOC.IEx.html#p1e/1"},{"type":"function","title":"AOC.IEx.p1i/1","doc":"Call part 1 of the current puzzle with its input.\n\nUses `p1/2` and `input_string/1` to call `Y .D .p1/1` with the input of `year` and\n`day`.\n\nIf not present in the options list, `day` and `year` are fetched from the application\nenvironment or based on the local time. Refer to the module documentation for additional\ninformation.\n\nThe `time:` option can be used to control if the execution of `p1` will be timed. If it is not\npresent, the value of `:time_calls?` in the application environment (default: `false`) will be\nrespected.\n\nThis function may cause recompilation if `auto_compile?` is enabled.","ref":"AOC.IEx.html#p1i/1"},{"type":"function","title":"AOC.IEx.p2/2","doc":"Call part 2 of the current puzzle with the given input.\n\nCalls, `Y .D .p2/1` with `input`.\n\nIf not present in the options list, `day` and `year` are fetched from the application\nenvironment or based on the local time. Refer to the module documentation for additional\ninformation.\n\nThe `time:` option can be used to control if the execution of `p2` will be timed. If it is not\npresent, the value of `:time_calls?` in the application environment (default: `false`) will be\nrespected.\n\nThis function may cause recompilation if `auto_compile?` is enabled.","ref":"AOC.IEx.html#p2/2"},{"type":"function","title":"AOC.IEx.p2e/1","doc":"Call part 2 of the current puzzle with its example input.\n\nUses `p2/2` and `example_string/1` to call `Y .D .p2/1` with the example input of\n`year` and `day`.\n\nIf not present in the options list, `day` and `year` are fetched from the application\nenvironment or based on the local time. Refer to the module documentation for additional\ninformation.\n\nThe `time:` option can be used to control if the execution of `p2` will be timed. If it is not\npresent, the value of `:time_calls?` in the application environment (default: `false`) will be\nrespected.\n\nThis function may cause recompilation if `auto_compile?` is enabled.","ref":"AOC.IEx.html#p2e/1"},{"type":"function","title":"AOC.IEx.p2i/1","doc":"Call part 2 of the current puzzle with its input.\n\nUses `p2/2` and `input_string/1` to call `Y .D .p2/1` with the input of `year` and\n`day`.\n\nIf not present in the options list, `day` and `year` are fetched from the application\nenvironment or based on the local time. Refer to the module documentation for additional\ninformation.\n\nThe `time:` option can be used to control if the execution of `p2` will be timed. If it is not\npresent, the value of `:time_calls?` in the application environment (default: `false`) will be\nrespected.\n\nThis function may cause recompilation if `auto_compile?` is enabled.","ref":"AOC.IEx.html#p2i/1"},{"type":"task","title":"mix aoc","doc":"Create a code skeleton and fetch input for the AOC challenge of a day / year.\n\nThis mix task runs `mix aoc.gen` followed by `mix aoc.get`. Afterwards, it prints the url of\ntoday's challenge. Please refer to `mix help aoc.gen` and `mix help aoc.get` for more\ninformation.\n\n## `mix aoc.gen`\n\nThis task creates a code skeleton for the AOC challenge of a given day / year. The generated\ncode is written to `lib/ / .ex`. Optionally, a unit test file is generated in\n`test/ / _test.exs`. The location where files are created can be customized.\n\n## `mix aoc.get`\n\nThis task fetches the input and example input for a given day and year and stores it in\n`input/ _ .txt` and `input/ _ _example.txt`. In order for this task to work,\nyour session cookie should be passed as a command line argument or set up in the\n`:advent_of_code_utils` application environment.","ref":"Mix.Tasks.Aoc.html"},{"type":"task","title":"Configuration - mix aoc","doc":"#","ref":"Mix.Tasks.Aoc.html#module-configuration"},{"type":"task","title":"Application environment - mix aoc","doc":"The following [application configuration parameters](`Config`) can modify the behaviour of this\ntask:\n\n- `day`: Specify the day. Defaults to the current day.\n- `year`: Specify the year. Defaults to the current year.\n- `time_zone`: Specify the time-zone used to determine the local time. Defaults to the time zone\n  of your computer.  Please refer to the [README](readme.html#time-zones) for additional\n  information.\n- `session`: Advent of code sessions cookie. Needed to allow `mix aoc.get` to fetch your\n  personal puzzle input.\n- `input_path`: Determines where `mix aoc.get` stores the input file. Defaults to\n  `\"input/:year_:day.txt\"`\n- `example_path`: Determines where `mix aoc.get` stores the example input. Defaults to\n`\"input/:year_:day_example.txt\"`\n- `gen_tests?`: Determines if `mix aoc.gen` creates test files. Defaults to `false`.\n- `gen_doctests?`: Determines if `mix aoc.gen` creates doctests. Defaults to the value of\n  `gen_tests?`.\n- `code_path`: Determines where `mix aoc.gen` stores the generated code file. Defaults to\n  `\"lib/:year/:day.ex\"`.\n- `code_path`: Determines where `mix aoc.gen` stores the generated test file. Defaults to\n`\"test/:year/:day_test.exs\"`\n\n#","ref":"Mix.Tasks.Aoc.html#module-application-environment"},{"type":"task","title":"Command-line arguments - mix aoc","doc":"- `-y` or `--year`: Specify the year.\n- `-d` or `--day`: Specify the day.\n- `-s` or `--session`: Specify the session cookie.\n- `--example`: Fetch example input (the default)\n- `--no-example`: Do not fetch example input\n- `-t` or `--test`: Generate tests.\n- `--no-test`: Do not generate tests.\n- `--doctest` or `--no-doctest`: Enable or disable the creation of doctests.\n\nAll of these options take precedence over their application environment counterparts.","ref":"Mix.Tasks.Aoc.html#module-command-line-arguments"},{"type":"task","title":"mix aoc.gen","doc":"Create a code skeleton for the AOC challenge of a given day / year.\n\nThis mix task creates a code skeleton for the advent of code challenge of a specific day. The\nday and year of the challenge can be passed as command-line arguments or be set in the\n`advent_of_code_utils` application configuration. When neither of these are present, the current\ndate is used.\n\n## (doc)tests\n\nBy default, this task only generates a solution module file. When `gen_tests?` is set to `true`\nin the application configuration (i.e. in `config/config.exs`), or when `--test` is passed as an\noption to this task, this task will also generate a unit test file. Moreover, it will also add\ndoctests to the solution module.\n\nIf you do not wish to generate doctests, you can set `gen_doctests?` to `false` in the\napplication configuration or pass the `--no-doctest` option to this task.\n\n#","ref":"Mix.Tasks.Aoc.Gen.html"},{"type":"task","title":"Generating tests later on - mix aoc.gen","doc":"This task will never overwrite existing files. Therefore, if you run `mix aoc.gen --test` after\na previous run of `mix aoc.gen` (without `--test`), this task will create a unit test file and\nleave the existing file alone. This is useful if you wish to add tests to existing code.\n\nNote that adding the `--doctest` flag when a solution module has already been generated will not\nwork, as this task will never overwrite existing files.","ref":"Mix.Tasks.Aoc.Gen.html#module-generating-tests-later-on"},{"type":"task","title":"File locations - mix aoc.gen","doc":"By default, this task stores the generated code skeleton in `lib/ / .ex`. If this file\nalready exists, an error is returned and no files are created. The destination path can be\nmodified by setting the value of `:code_path` in the `advent_of_code_utils` application\nconfiguration. This value should be set to a string which may contain `:year` and `:day`. These\nvalues will be replaced by the day and year for which a skeleton is generated.\n\nFor instance, the following configuration will store the generated code in\n`lib/aoc_ _day_ .ex`:\n\n```\nconfig :advent_of_code_utils, :code_path, \"lib/aoc_:year_day_:day.ex\"\n```\n\nIf tests are generated, they are stored in `test/ / _test.exs`. This value can be\nmodified by setting `test_path` in the application configuration. Note that, in order for the\ntest to work with `mix test`, the test path must end in `_test.exs`.","ref":"Mix.Tasks.Aoc.Gen.html#module-file-locations"},{"type":"task","title":"Configuration - mix aoc.gen","doc":"#","ref":"Mix.Tasks.Aoc.Gen.html#module-configuration"},{"type":"task","title":"Application environment - mix aoc.gen","doc":"The following [application configuration parameters](`Config`) can modify the behaviour of this\ntask:\n\n- `day`: Specify the day. Defaults to the current day.\n- `year`: Specify the year. Defaults to the current year.\n- `time_zone`: Specify the time-zone used to determine the local time. Defaults to the time zone\n  of your computer.  Please refer to the [README](readme.html#time-zones) for additional\n  information.\n- `gen_tests?`: Determines if test files are created. Defaults to `false`.\n- `gen_doctests?`: Determines if doctests are generated. Defaults to the value of `gen_tests?`.\n- `code_path`: Determines where the generated code file is stored. Defaults to\n  `\"lib/:year/:day.ex\"`.\n- `code_path`: Determines where the generated test file is stored. Defaults to\n  `\"test/:year/:day_test.exs\"`\n\n#","ref":"Mix.Tasks.Aoc.Gen.html#module-application-environment"},{"type":"task","title":"Command-line arguments - mix aoc.gen","doc":"The options below take precedence over values defined in the application configuration.\n\n- `-d` or `--day`: Specify the day.\n- `-y` or `--year`: Specify the year.\n- `-t` or `--test`: Generate tests.\n- `--no-test`: Do not generate tests.\n- `--doctest` or `--no-doctest`: Enable or disable the creation of doctests.","ref":"Mix.Tasks.Aoc.Gen.html#module-command-line-arguments"},{"type":"function","title":"Mix.Tasks.Aoc.Gen.run_task/1","doc":"","ref":"Mix.Tasks.Aoc.Gen.html#run_task/1"},{"type":"task","title":"mix aoc.get","doc":"Fetch the input and example input of the AOC challenge for a given day / year.\n\nThis mix task fetches the input and example input for the advent of code challenge of a specific\nday. The day and year of the challenge can be passed as command-line arguments or be set in the\n`advent_of_code_utils` application configuration. When neither is present, the current date is\nused.\n\nBy default, this task stores the fetched input data in `input/ _ .txt`. The fetched\nexample is stored in `input/ _ _example.txt`. If a file already exists, the matching\ninput is not fetched. The destination paths can be modified by setting the value of\n`:input_path` or `:example_path` in the `advent_of_code_utils` application configuration. These\nvalues should be set to a string which may contain `:year` and `:day`. These values will be\nreplaced by the day and year of which the input is fetched.\n\nFor instance, the following configuration will store the fetched input data in\n`my_input/ / .input`:\n\n```\nconfig :advent_of_code_utils, :input_path, \"my_input/:year/:day.input\"\n```","ref":"Mix.Tasks.Aoc.Get.html"},{"type":"task","title":"Session cookie - mix aoc.get","doc":"In order to fetch your input, this task needs your advent of code session cookie. This can be\nobtained by investigating your cookies after logging in on the advent of code website. The\ncookie can be stored inside your `config/config.exs` file (e.g.\n`config, :advent_of_code_utils, :session, \" \"`) or it can be passed as a\ncommand-line argument.  If no cookie is present, the input cannot be fetched.","ref":"Mix.Tasks.Aoc.Get.html#module-session-cookie"},{"type":"task","title":"Example input - mix aoc.get","doc":"The example input of a given day is fetched by parsing the challenge webpage of a given day and\nreturning the first code example found on that page. This is generally the example input of that\nday. As this method is not foolproof, it is sometimes necessary to modify the example file by\nhand.\n\nIf you do not wish to fetch example input, you can pass the `--no-example` flag to this task, or\nyou can set `fetch_example?` to `false` in the `advent_of_code_utils` application configuration.","ref":"Mix.Tasks.Aoc.Get.html#module-example-input"},{"type":"task","title":"Configuration - mix aoc.get","doc":"#","ref":"Mix.Tasks.Aoc.Get.html#module-configuration"},{"type":"task","title":"Application environment - mix aoc.get","doc":"The following [application configuration parameters](`Config`) can modify the behaviour of this\ntask:\n\n- `session`: Advent of code sessions cookie.\n- `day`: Specify the day. Defaults to the current day.\n- `year`: Specify the year. Defaults to the current year.\n- `time_zone`: Specify the time-zone used to determine the local time. Defaults to the time zone\n  of your computer.  Please refer to the [README](readme.html#time-zones) for additional\n  information.\n- `input_path`: Determines where the input file is stored. Defaults to `\"input/:year_:day.txt\"`\n- `example_path`: Determines where the example input is stored. Defaults to\n  `\"input/:year_:day_example.txt\"`\n\n#","ref":"Mix.Tasks.Aoc.Get.html#module-application-environment"},{"type":"task","title":"Command-line arguments - mix aoc.get","doc":"The options below take precedence over values defined in the application configuration.\n\n- `-s` or `--session`: Specify the session cookie.\n- `-d` or `--day`: Specify the day.\n- `-y` or `--year`: Specify the year.\n- `--no-example`: Do not fetch example input\n- `--example`: Fetch example input","ref":"Mix.Tasks.Aoc.Get.html#module-command-line-arguments"},{"type":"function","title":"Mix.Tasks.Aoc.Get.run_task/1","doc":"","ref":"Mix.Tasks.Aoc.Get.html#run_task/1"},{"type":"extras","title":"Advent of Code Utils","doc":"# Advent of Code Utils\n\n[![hex.pm](https://img.shields.io/hexpm/v/advent_of_code_utils.svg)](https://hex.pm/packages/advent_of_code_utils)\n[![hexdocs.pm](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/advent_of_code_utils/)\n[![hex.pm](https://img.shields.io/hexpm/l/advent_of_code_utils.svg)](https://hex.pm/packages/advent_of_code_utils)\n\nInput fetching and boilerplate generation for [Advent of Code](https://adventofcode.com/).\n\nThe goal of this project is to eliminate most of the manual labor involved with\nworking on the yearly Advent of Code challenges.\n\nAs a sample, this is the workflow you'd use when working on the challenge of the\nfirst of December 2020:\n\n```\n$ mix aoc\n* Creating: lib/2020/1.ex\n* Creating: input/2020_1.txt\n* Creating: input/2020_1_example.txt\nToday's challenge can be found at: https://adventofcode.com/2020/day/1\n```\n\nAfterwards, `lib/2020/1.ex` will look as follows:\n\n```elixir\nimport AOC\n\naoc 2020, 1 do\n  def p1(input) do\n  end\n\n  def p2(input) do\n  end\nend\n```\n\nWhile solving your challenge, you can use the `AOC.IEx.p1e/1` and\n`AOC.IEx.p2e/1` helpers in `iex` to test your solution so far with the example\ninput. Once ready, you can use `AOC.IEx.p1i/1` and `AOC.IEx.p2i/1` to run your\nsolution on your puzzle input.\n\nThe project also optionally supports automatically recompiling your mix project\nwhen using the aforementioned helpers, timing your solutions, and boilerplate\ngeneration for unit testing your solution modules. It also supports working on\npuzzles from previous days or years. Check out the\n[docs](https://hexdocs.pm/advent_of_code_utils/) or the\n[cheatsheet](https://hexdocs.pm/advent_of_code_utils/cheatsheet.html) for more\ninformation.","ref":"readme.html"},{"type":"extras","title":"Setup & Use - Advent of Code Utils","doc":"- Add `advent_of_code_utils` to your list of dependencies in `mix.exs`:\n  ```elixir\n  def deps do\n    [\n      {:advent_of_code_utils, \"~> 4.0\"}\n    ]\n  end\n  ```\n\n- Configure your Advent of Code project in `config/config.exs`:\n\n  - Store your session cookie. You can find this by inspecting your cookies\n    after logging in to the advent of code website.\n\n    ```elixir\n    config :advent_of_code_utils, session: \" \"\n    ```\n\n  - _(Optional)_ Set `auto_compile?` to `true` if you want the various `AOC.IEx.p*` helpers\n    to recompile your project:\n\n    ```elixir\n    config :advent_of_code_utils, auto_compile?: true\n    ```\n\n  - _(Optional)_ Set `time_calls?` to `true` if you want the various `AOC.IEx.p*` helpers\n    to show the runtime of calling a solution.\n\n    ```elixir\n    config :advent_of_code_utils, time_calls?: true\n    ```\n\n  - _(Optional)_ Set `gen_tests?` to `true` if you want `mix aoc.gen` to\n    generate unit test code.\n\n    ```elixir\n    config :advent_of_code_utils, gen_tests?: true\n    ```\n\n  - _(Optional)_ Configure `iex` to display charlists as lists. This will prevent lists like\n    `[99, 97, 116]` to show up as `'cat'`:\n\n    ```elixir\n    config :iex, inspect: [charlists: :as_lists]\n    ```\n\n  - If you follow these steps, your `config/config.exs` should look as follows:\n\n    ```elixir\n    import Config\n\n    config :advent_of_code_utils,\n      auto_compile?: true,\n      time_calls?: true,\n      gen_tests?: true,\n      session: \" \"\n\n    config :iex,\n      inspect: [charlists: :as_lists]\n    ```\n\n- _(Optional)_ Add `import AOC.IEx` to your\n  [`.iex.exs` file](https://hexdocs.pm/iex/IEx.html#module-the-iex-exs-file).\n  This allows you to use the utilities defined in `AOC.IEx` without\n  specifying the module name.\n\n- _(Optional)_ Add `input/` to your `.gitignore` file if you use git.\n\nNow that you are set up, you can use `mix aoc` to work on today's challenge.\nThe day and year of a challenge can be passed in various ways, so this project\ncan still be used when working on older challenges.\n\nIf you only want to use this application to fetch the input of a challenge\nwithout generating any code, you can skip most of the optional steps above and\nuse `mix aoc.get` instead of `mix aoc`.","ref":"readme.html#setup-use"},{"type":"extras","title":"Example Input - Advent of Code Utils","doc":"Besides fetching input, `mix aoc.get` and `mix aoc` will also fetch example\ninput for the given day.  This is done by reading the first code example on the\nchallenge webpage, which is generally that day's example input.\n\nSince this method is not 100% reliable, you may you wish to disable this\nbehaviour. This can be done by passing the `--no-example` flag to `mix aoc` or\n`mix aoc.get` or by setting `fetch_example?` to false in your `config.exs`\nfile.","ref":"readme.html#example-input"},{"type":"extras","title":"Time Zones - Advent of Code Utils","doc":"By default, this project uses your system time (as determined by\n`NaiveDateTime.local_now/0`), to determine the current \"day\". Said otherwise,\nif your computer says it is currently the 8th of December, `mix aoc.get`,\n`AOC.IEx.p1/2`, and other functions which reason about time will assume it is\nthe 8th day of December. This can be problematic if you live in a time zone\nwhich does not align well with the publication time of the AOC puzzles\n(midnight US Eastern Standard Time).\n\nThis problem can be solved by setting the `time_zone` option of\n`advent_of_code_utils`. When this option is set to `:aoc`, the project will\ndetermine the current day based on EST, the time zone of the advent of code\nservers. When it is said to `:local` (the default), your system time will be\nused. Alternatively, a valid time zone string can be supplied, in which case\nthe project will determine the current day based on the provided time zone.\n\n```elixir\n# Use the aoc timezone instead of local time\nconfig :advent_of_code_utils, time_zone: :aoc\n```","ref":"readme.html#time-zones"},{"type":"extras","title":"Issues - Advent of Code Utils","doc":"This project grew from a collection of utilities I wrote for myself when working\non advent of code.\nI polished these utilities, but it is possible some bugs are still present.\nIf you run into any issue, feel free to create an issue on\n[GitHub](https://github.com/mathsaey/advent_of_code_utils).","ref":"readme.html#issues"},{"type":"extras","title":"AoC Automation Guidelines - Advent of Code Utils","doc":"This tool follows the [automation guidelines](https://www.reddit.com/r/adventofcode/wiki/faqs/automation)\non the /r/adventofcode community wiki. The following information is intended to\nspecify how we follow these guidelines:\n\n* All code that communicates with the AoC servers is located in\n  `lib/mix/tasks/aoc.get.ex`.\n* This tool only communicates with the AoC servers at the request of the user\n  (i.e. when they use `mix aoc.get` or `mix aoc`).\n* Fetched inputs are stored locally and are never retrieved again unless the\n  user deletes the retrieved file manually (`do_if_file_does_not_exist/2`).\n* The `User-Agent` header is set in `fetch/2` and contains my personal contact\n  information and a reference to this repository.","ref":"readme.html#aoc-automation-guidelines"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"4.0.0 - Changelog","doc":"#","ref":"changelog.html#4-0-0"},{"type":"extras","title":"Overview - Changelog","doc":"##","ref":"changelog.html#overview"},{"type":"extras","title":"Dropping support for helper functions in solution modules - Changelog","doc":"This update makes the \"puzzle input\" method introduced in 3.0.0 the only way\nto write solution modules. The old, \"helper function\" method is no longer\nsupported.\n\nThe helper function method worked as follows:\n\n```\naoc year, day do\n  def p1, do: input_string() |> # Solve puzzle\nend\n```\n\nThe puzzle input method works as follows:\n\n```\naoc year, day do\n  def p1(input), do: input |> # solve puzzle\nend\n```\n\nInput is provided through the use of the helpers in `AOC.IEx`.\n\nThe input method is preferred as it makes it easier to test your solution using\ndifferent inputs without the need to modify any code. It also makes it\nsignificantly easier to write tests for your solution module.\n\nUsers who wish to keep using the helper method are advised to stick to version\n3 of Advent of Code Utils.\n\n#### `aoc_test/4`\n\nThis update adds optional support for generating unit test modules for your\nsolution modules. You can now write:\n\n```\naoc_test year, day do\nend\n```\n\nWhich will create an `ExUnit.Case` which tests your solution module. The\ngenerated case will automatically call the doctests of your solution module.\n\n`mix aoc.gen` can now also generate this test code and add doctests to your\nsolution module.\n\n#","ref":"changelog.html#dropping-support-for-helper-functions-in-solution-modules"},{"type":"extras","title":"Detailed Changelog - Changelog","doc":"- Introduce `AOC.aoc_test/4`, `AOC.Case`.\n- Update `mix aoc.gen` to generate unit test files\n- Remove `example_path/1`, `input_path/1`, `example_string/1`,\n  `input_string/1`, `example_stream/1`, `input_stream/1` from `AOC` module.\n- Remove `example_stream/1`, `input_stream/1` from `AOC.IEx`.\n- Rename `fetch_example` configuration to `fetch_example?` for consistency.","ref":"changelog.html#detailed-changelog"},{"type":"extras","title":"3.1.3 - Changelog","doc":"- Optionally show the elapsed time when calling a `p1`, `p2`, `p1i`, `p1e`,\n  `p2i` and `p2e`. This is done when the `time_calls?` configuration option is\n  set to true.","ref":"changelog.html#3-1-3"},{"type":"extras","title":"3.1.2 - Changelog","doc":"- `AOC.IEx.mod/1` now calls `Code.ensure_loaded!/1` to ensure the target module\n  is loaded. This is done to prevent issues where `p1e` and `p1i` complain that\n  the target module does not exist.","ref":"changelog.html#3-1-2"},{"type":"extras","title":"3.1.1 - Changelog","doc":"- `input_stream`, `input_string`, `example_stream` and `example_string` now\n  only trim trailing newlines, not significant whitespace.","ref":"changelog.html#3-1-1"},{"type":"extras","title":"3.1.0 - Changelog","doc":"- Timezone support: the project now accepts a `time_zone` configuration setting\n  which can be used to specify the time zone used by the input fetcher and iex\n  helpers. This can be set to a time zone string, to `:aoc` or to `:local` (the\n  default). `:local` uses the system time, as before, while `:aoc` uses the\n  advent of code time zone (i.e. EST).","ref":"changelog.html#3-1-0"},{"type":"extras","title":"3.0.0 - Changelog","doc":"- `AOC.IEx` has been reworked.\n  - All functions accept an `opts` keyword list used to specify year or day\n    when needed.\n  - `p1` and `p2` now accept puzzle input as an argument, making it easier to\n    switch between different inputs or examples.\n  - Introduce `p1e`, `p1i`, `p2e` and `p2i` to facilitate calling `p1` or `p2`\n    with example or puzzle input.\n  - Introduce wrapper functions `example_path/1`, `input_path/1`,\n    `example_string/1`, `input_string/1`, `example_stream/1` and\n    `input_stream/1` to facilitate experimentation inside iex.\n- Update `AOC` documentation to reflect to new workflow.\n- Update `mix aoc.gen` to generate a template more suited to new workflow.","ref":"changelog.html#3-0-0"},{"type":"extras","title":"2.0.2 - Changelog","doc":"- Pass user agent when using `mix aoc.get`","ref":"changelog.html#2-0-2"},{"type":"extras","title":"2.0.1 - Changelog","doc":"- Update ex_doc, add typespecs to public functions.","ref":"changelog.html#2-0-1"},{"type":"extras","title":"2.0.0 - Changelog","doc":"- `input_string` and `example_string` now both call `String.trim_trailing/1` on\n  the returned string.","ref":"changelog.html#2-0-0"},{"type":"extras","title":"1.1.0 - Changelog","doc":"- Download example input when using `mix aoc.get`.\n- Add `example_*` functions to `AOC` module.","ref":"changelog.html#1-1-0"},{"type":"extras","title":"1.0.1 - Changelog","doc":"- Make `input_*` functions public (`def` instead of `defp`) to\n  facilitate debugging.","ref":"changelog.html#1-0-1"},{"type":"extras","title":"1.0.0 - Changelog","doc":"- Initial stable release","ref":"changelog.html#1-0-0"},{"type":"extras","title":"Cheatsheet","doc":"# Cheatsheet\n\nThis document provides an at-a-glance overview on how to use \"Advent of Code\nUtils\". In-depth information on each topic can be found in the relevant docs.","ref":"cheatsheet.html"},{"type":"extras","title":"The Basics - Cheatsheet","doc":"{: .col-3}\n\n#","ref":"cheatsheet.html#the-basics"},{"type":"extras","title":"Solution modules - Cheatsheet","doc":"A solution for a day is written in a _solution module_. These modules are\nwritten using the `AOC.aoc/3` macro.\n\n```\nimport AOC\n\naoc  ,   do\n  def p1(input) do\n    # part 1 solution goes here\n  end\n\n  def p2(input) do\n    # part 2 solution goes here\n  end\n\n  def another_function do\n    # arbitrary code goes here\n  end\n\n  # any code that is valid in an\n  # elixir module is valid here\nend\n```\nThis generates a standard elixir module named `Y .D `.\n\n#","ref":"cheatsheet.html#solution-modules"},{"type":"extras","title":"iex interaction - Cheatsheet","doc":"Inside iex, you can use the functions described in `AOC.IEx` to test your\nsolution:\n\n##","ref":"cheatsheet.html#iex-interaction"},{"type":"extras","title":"Test solutions - Cheatsheet","doc":"- `AOC.IEx.p1e/1`\n- `AOC.IEx.p1i/1`\n- `AOC.IEx.p2e/1`\n- `AOC.IEx.p2i/1`\n- `AOC.IEx.p1/2`\n- `AOC.IEx.p2/2`\n\n##","ref":"cheatsheet.html#test-solutions"},{"type":"extras","title":"Get module (to test helpers) - Cheatsheet","doc":"- `AOC.IEx.mod/1`\n\n##","ref":"cheatsheet.html#get-module-to-test-helpers"},{"type":"extras","title":"Get puzzle / example data - Cheatsheet","doc":"- `AOC.IEx.input_path/1`\n- `AOC.IEx.example_path/1`\n- `AOC.IEx.input_string/1`\n- `AOC.IEx.example_string/1`\n#","ref":"cheatsheet.html#get-puzzle-example-data"},{"type":"extras","title":"Mix tasks - Cheatsheet","doc":"mix is used to fetch the input for a puzzle and to create boilerplate code.\n\n##","ref":"cheatsheet.html#mix-tasks"},{"type":"extras","title":"Fetch input / example - Cheatsheet","doc":"* `mix aoc.get`\n\n##","ref":"cheatsheet.html#fetch-input-example"},{"type":"extras","title":"Generate code skeleton - Cheatsheet","doc":"* `mix aoc.gen`\n\n##","ref":"cheatsheet.html#generate-code-skeleton"},{"type":"extras","title":"Fetch + Generate - Cheatsheet","doc":"* `mix aoc`","ref":"cheatsheet.html#fetch-generate"},{"type":"extras","title":"Specifying time and date - Cheatsheet","doc":"{: .col-2}\n\n#","ref":"cheatsheet.html#specifying-time-and-date"},{"type":"extras","title":"In iex - Cheatsheet","doc":"```\niex> p1i()\n# Runs p1 of the current day\n# with the input string of that day\niex> p1i(day: 8)\n# Runs p1 of day 8 of the current\n# year with the input string of that day\niex> p1i(day: 8, year: 1991)\n# Runs p1 of day 8 of 1991\n# with the input string of that day\niex> p1i(year: 1991)\n# Uses the current day of the month,\n# but in 1991\niex> p1(\"some input\", day: 8)\n# Runs p1 of day 8 of the current\n# year with the provided input string\n```\nA day can be specified like this for all `AOC.IEx` functions.\n\n#","ref":"cheatsheet.html#in-iex"},{"type":"extras","title":"Specifying year or day in `config.exs` - Cheatsheet","doc":"Year or day may also be specified in the `config/config.exs` file:\n\n```\nimport Config\n\nconfig :advent_of_code_utils,\n  day: 8,\n  year: 1991\n```\n\nSetting the year is, for instance, useful when going back to finish previous\nyears.\n\n#","ref":"cheatsheet.html#specifying-year-or-day-in-config-exs"},{"type":"extras","title":"When using mix tasks - Cheatsheet","doc":"* `mix aoc --day   --year  `\n  * Generates code + fetches input for the given year and day.\n* `mix aoc -d   -y  `\n  * Same as above, but shorter.\n* `mix aoc --day  `\n  * Generates code + fetches input for the current year with the given day.\n* `mix aoc --year  `\n  * Generates code + fetches input for the given year with the current day.\n\nThe same options can be passed to `mix aoc.gen` and `mix aoc.get`.\n\n#","ref":"cheatsheet.html#when-using-mix-tasks"},{"type":"extras","title":"Precedence - Cheatsheet","doc":"Mix tasks and the `AOC.IEx` helpers use the year and date from the following\nsources, using the first that matches.\n\n1. Options passed as argument to a function or as a flag to a mix task.\n2. Year or day as specified in the application configuration (i.e. in\n   `config/config.exs`).\n3. The local year and month of day.","ref":"cheatsheet.html#precedence"},{"type":"extras","title":"Tests - Cheatsheet","doc":"{: .col-2}\n\nAdvent of Code utils offers optional support for generating and\nwriting (doc)tests.\n\n#","ref":"cheatsheet.html#tests"},{"type":"extras","title":"Test modules - Cheatsheet","doc":"`AOC.aoc_test/4` can be used to generate a unit test module for a solution\nmodule.\n\n```\nimport AOC\n\naoc_test  ,   do\n  # test code goes here\nend\n```\nThis generates an elixir module named `Y .D .AOCTest` which `use`s\n`AOC.Case`.\n\nThis code can be generated by `mix aoc.gen`.\n\n#","ref":"cheatsheet.html#test-modules"},{"type":"extras","title":"accessing (puzzle) input - Cheatsheet","doc":"Inside the `aoc_test` module or inside the doctests, you can access the puzzle\ninput and the example input using the following functions:\n\n- `input_string/0`\n- `example_string/0`\n- `input_path/0`\n- `example_path/0`\n\n`AOC.aoc_test/4` also `import/2`s the solution module, so you can access any\nof its functions as well.\n\n#","ref":"cheatsheet.html#accessing-puzzle-input"},{"type":"extras","title":"doctests - Cheatsheet","doc":"`AOC.aoc_test/4` automatically calls the doctests of the solution module.\nThis enables you to write your solution module as follows:\n```\naoc  ,   do\n  @doc \"\"\"\n      iex> p1(example_input())\n      # solution to part 1 here\n  \"\"\"\n  def p1(input), do: ...\nend\n```\n`mix aoc.gen` can automatically generate doctests skeletons for your solution\nmodule.\n\n#","ref":"cheatsheet.html#doctests"},{"type":"extras","title":"Running the tests - Cheatsheet","doc":"`AOC.aoc_test/4` injects several module tags into the test case, which can be\nused to make `mix test` only run the (doc)tests you want. See `AOC.Case` for\nmore infomation.\n\n##","ref":"cheatsheet.html#running-the-tests"},{"type":"extras","title":"Run the test of a specific day - Cheatsheet","doc":"* `mix test --only day: `\n##","ref":"cheatsheet.html#run-the-test-of-a-specific-day"},{"type":"extras","title":"Run the test of a specific day if your repository spans multiple years - Cheatsheet","doc":"* `mix test --only aoc: - `\n* `mix test test/ / _test.exs`\n##","ref":"cheatsheet.html#run-the-test-of-a-specific-day-if-your-repository-spans-multiple-years"},{"type":"extras","title":"Run all tests of a year - Cheatsheet","doc":"* `mix test --only year: `\n##","ref":"cheatsheet.html#run-all-tests-of-a-year"},{"type":"extras","title":"Run all tests - Cheatsheet","doc":"* `mix test`","ref":"cheatsheet.html#run-all-tests"},{"type":"extras","title":"Configuration & Setup - Cheatsheet","doc":"We recommend going through the full [Setup & Use](readme.html#setup-use)\nsection of the [readme](readme.html).\n\n#","ref":"cheatsheet.html#configuration-setup"},{"type":"extras","title":"Available configuration options - Cheatsheet","doc":"| Option | Used by | Default |\n| ------ | ------- | ------- |\n| `day` | `AOC.IEx` `mix aoc.get` `mix aoc.gen` | Current day of month |\n| `year` | `AOC.IEx` `mix aoc.get` `mix aoc.gen` | Current year |\n| `time_zone` | `AOC.IEx` `mix aoc.get` `mix aoc.gen` | `:local` |\n| `session` | `mix aoc.get` | |\n| `auto_compile?` | `AOC.IEx` | `false` |\n| `time_calls?` | `AOC.IEx` | `false` |\n| `gen_tests?` | `mix aoc.gen` | `false` |\n| `gen_doctests?` | `mix aoc.gen` | value of `gen_tests?` |\n| `code_path` | `mix aoc.gen` | `\"lib/:year/:day.ex\"` |\n| `test_path` | `mix aoc.gen` | `\"test/:year/:day_test.exs\"` |\n| `input_path` | `AOC.IEx` `mix aoc.get` | `\"input/:year_:day.txt\"` |\n| `example_path` | `AOC.IEx` `mix aoc.get` | `\"input/:year_:day_example.txt\"` |\n\n#","ref":"cheatsheet.html#available-configuration-options"},{"type":"extras","title":"Other recommended tweaks - Cheatsheet","doc":"* Add `import AOC.IEx` to\n  [the `.iex.exs` file](https://hexdocs.pm/iex/IEx.html#module-the-iex-exs-file).\n  This allows you to call `AOC.IEx.p1i/1` and similar in iex without the need\n  to prefix it.\n* Add `config :iex, inspect: [charlists: :as_lists]` to `config/config.exs`.\n  This prevents elixir from displaying lists of integers as strings.","ref":"cheatsheet.html#other-recommended-tweaks"}],"content_type":"text/markdown"}